from __future__ import annotations

import asyncio
from typing import TYPE_CHECKING

from nonechat.app import Frontend
from nonechat.setting import ConsoleSetting
from textual.color import Color

from avilla.standard.core.account import AccountUnavailable
from launart import Launart, Service

from .backend import AvillaConsoleBackend

if TYPE_CHECKING:
    from .protocol import ConsoleProtocol


class ConsoleService(Service):
    id = "console.service"
    required: set[str] = set()
    stages: set[str] = {"preparing", "blocking", "cleanup"}
    app: Frontend[AvillaConsoleBackend]
    protocol: ConsoleProtocol

    def __init__(self, protocol: ConsoleProtocol):
        self.app = Frontend(
            AvillaConsoleBackend,
            ConsoleSetting(
                title="Console",
                sub_title="Welcome to Avilla",
                icon="Avilla",
                icon_color=Color.parse("#22b14c"),
                title_color=Color(229, 192, 123, 1),
                bg_color=Color(40, 44, 52, 1),
                header_color=Color(90, 99, 108, 0.6),
                user_avatar="😃",
            ),
        )
        self.app.backend.set_service(self)
        self.protocol = protocol
        super().__init__()

    async def launch(self, manager: Launart):
        async with self.stage("preparing"):
            ...

        async with self.stage("blocking"):
            task = asyncio.get_running_loop().create_task(self.app.run_async())
            await manager.status.wait_for_sigexit()
        async with self.stage("cleanup"):
            self.app.exit()
            if task:
                await task
            self.protocol.avilla.broadcast.postEvent(AccountUnavailable(self.protocol.avilla, self.app.backend.account))



from dataclasses import dataclass, field
from typing import Optional, Union

from rich.console import JustifyMethod
from rich.emoji import EmojiVariant
from rich.style import Style
from rich.text import Text as RichText

from graia.amnesia.message.element import Element


class Emoji(Element):
    name: str

    def __init__(self, name: str):
        self.name = name

    def __str__(self):
        return f"[$Emoji:name={self.name}]"


@dataclass
class Markup(Element):
    markup: str
    style: Union[str, Style] = field(default="none")
    emoji: bool = field(default=True)
    emoji_variant: Optional[EmojiVariant] = field(default=None)

    def __str__(self):
        return f"[$Markup:markup={self.markup}]"


@dataclass
class Markdown(Element):
    markup: str
    code_theme: str = field(default="monokai")
    justify: Optional[JustifyMethod] = field(default=None)
    style: Union[str, Style] = field(default="none")
    hyperlinks: bool = field(default=True)
    inline_code_lexer: Optional[str] = field(default=None)
    inline_code_theme: Optional[str] = field(default=None)

    def __str__(self) -> str:
        return f"[$Markdown:markup={RichText.from_markup(self.markup, style=self.style, end='')}]"



from __future__ import annotations

from typing import TYPE_CHECKING

from avilla.core.ryanvk.collector.account import AccountCollector
from avilla.core.selector import Selector
from avilla.standard.core.activity import ActivityTrigger

if TYPE_CHECKING:
    from avilla.console.account import ConsoleAccount  # noqa
    from avilla.console.protocol import ConsoleProtocol  # noqa


class ConsoleActivityActionPerform((m := AccountCollector["ConsoleProtocol", "ConsoleAccount"]())._):
    m.post_applying = True

    @ActivityTrigger.trigger.collect(m, "land.user.activity(bell)")
    async def bell(self, target: Selector | None = None):
        await self.account.client.call("bell", {})



from __future__ import annotations

from dataclasses import asdict
from typing import TYPE_CHECKING

from nonechat.message import Element
from nonechat.message import Emoji as CslEmoji
from nonechat.message import Markdown as CslMarkdown
from nonechat.message import Markup as CslMarkup
from nonechat.message import Text as CslText

from avilla.console.element import Emoji, Markdown, Markup
from avilla.core.elements import Text
from avilla.core.ryanvk.collector.application import ApplicationCollector
from avilla.core.ryanvk.descriptor.message.deserialize import MessageDeserialize

ConsoleMessageDeserialize = MessageDeserialize[Element]


class ConsoleMessageDeserializePerform((m := ApplicationCollector())._):
    m.post_applying = True

    # LINK: https://github.com/microsoft/pyright/issues/5409

    @ConsoleMessageDeserialize.collect(m, "Text")
    async def text(self, element: Element) -> Text:
        if TYPE_CHECKING:
            assert isinstance(element, CslText)
        return Text(element.text)

    @ConsoleMessageDeserialize.collect(m, "Emoji")
    async def emoji(self, element: Element) -> Emoji:
        if TYPE_CHECKING:
            assert isinstance(element, CslEmoji)
        return Emoji(element.name)

    @ConsoleMessageDeserialize.collect(m, "Markup")
    async def markup(self, element: Element) -> Markup:
        if TYPE_CHECKING:
            assert isinstance(element, CslMarkup)
        return Markup(**asdict(element))

    @ConsoleMessageDeserialize.collect(m, "Markdown")
    async def markdown(self, element: Element) -> Markdown:
        if TYPE_CHECKING:
            assert isinstance(element, CslMarkdown)
        return Markdown(**asdict(element))



from __future__ import annotations

from typing import TYPE_CHECKING

from nonechat.info import Event
from nonechat.message import Element

from avilla.core.account import AccountStatus, BaseAccount
from avilla.core.platform import Abstract, Land, Platform
from avilla.core.selector import Selector

if TYPE_CHECKING:
    from .protocol import ConsoleProtocol

PLATFORM = Platform(
    Land(
        "console",
        [{"name": "GraiaxCommunity"}],
        humanized_name="Avilla-Console - Console Impl for avilla",
    ),
    Abstract(
        protocol="Console",
        maintainers=[{"name": "yanyongyu"}],
        humanized_name="Textual Console",
    ),
)


class ConsoleAccount(BaseAccount):
    protocol: ConsoleProtocol
    status: AccountStatus

    def __init__(self, protocol: ConsoleProtocol):
        super().__init__(Selector().land("console").account(protocol.name), protocol.avilla)
        self.protocol = protocol
        self.status = AccountStatus()

    @property
    def client(self):
        return self.protocol.service.app

    @property
    def available(self) -> bool:
        return self.status.enabled

    def __staff_generic__(self, element_type: Element, event_type: Event):
        ...



from __future__ import annotations

from datetime import datetime
from secrets import token_hex
from typing import TYPE_CHECKING

from nonechat.info import Event, MessageEvent

from avilla.core.context import Context
from avilla.core.message import Message
from avilla.core.ryanvk.collector.account import AccountCollector
from avilla.core.ryanvk.descriptor.event import EventParse
from avilla.core.ryanvk.staff import Staff
from avilla.core.selector import Selector
from avilla.standard.core.message import MessageReceived

if TYPE_CHECKING:
    from avilla.console.account import ConsoleAccount  # noqa
    from avilla.console.protocol import ConsoleProtocol  # noqa

ConsoleEventParse = EventParse[Event]


class ConsoleEventMessagePerform((m := AccountCollector["ConsoleProtocol", "ConsoleAccount"]())._):
    m.post_applying = True

    @ConsoleEventParse.collect(m, "console.message")
    async def console_message(self, raw_event: Event):
        if TYPE_CHECKING:
            assert isinstance(raw_event, MessageEvent)
        message = await Staff.focus(self.account, element_typer=lambda e: type(e).__name__).deserialize_message(
            raw_event.message.content
        )
        console = Selector().land(self.account.route["land"]).user(str(raw_event.user.id))
        context = Context(
            account=self.account,
            client=console,
            endpoint=console,
            scene=console,
            selft=self.account.route,
        )
        return MessageReceived(
            context,
            Message(
                id=token_hex(16),
                scene=console,
                sender=console,
                content=message,
                time=datetime.fromtimestamp(raw_event.time.timestamp()),
            ),
        )



from __future__ import annotations

from secrets import token_hex
from typing import TYPE_CHECKING

from loguru import logger
from nonechat.info import Robot
from nonechat.message import ConsoleMessage

from avilla.core.ryanvk.collector.account import AccountCollector
from avilla.core.ryanvk.staff import Staff
from avilla.core.selector import Selector
from avilla.standard.core.message import MessageSend
from graia.amnesia.message import MessageChain

if TYPE_CHECKING:
    from avilla.console.account import ConsoleAccount  # noqa
    from avilla.console.protocol import ConsoleProtocol  # noqa


class ConsoleMessageActionPerform((m := AccountCollector["ConsoleProtocol", "ConsoleAccount"]())._):
    m.post_applying = True
    targets = ["land.user"]

    @MessageSend.send.collect(m, "land.user")
    async def send_console_message(
        self,
        target: Selector,
        message: MessageChain,
        *,
        reply: Selector | None = None,
    ) -> Selector:
        if TYPE_CHECKING:
            assert isinstance(self.protocol, ConsoleProtocol)
        serialized_msg = ConsoleMessage(await Staff.focus(self.account).serialize_message(message))

        await self.account.client.call("send_msg", {"message": serialized_msg, "info": Robot(self.protocol.name)})
        logger.info(f"{self.account.route['land']}: [send]" f"[Console]" f" <- {str(message)!r}")
        return Selector().land(self.account.route["land"]).message(token_hex(16))



from __future__ import annotations

from typing import TYPE_CHECKING

from avilla.core.ryanvk.collector.account import AccountCollector
from avilla.core.selector import Selector
from avilla.standard.core.profile import Nick, Summary

if TYPE_CHECKING:
    from avilla.console.account import ConsoleAccount  # noqa
    from avilla.console.protocol import ConsoleProtocol  # noqa


class ConsoleProfileActionPerform((m := AccountCollector["ConsoleProtocol", "ConsoleAccount"]())._):
    m.post_applying = True

    @m.pull("lang.user", Nick)
    async def get_console_nick(self, target: Selector) -> Nick:
        console = self.account.client.storage.current_user
        return Nick(console.nickname, console.nickname, "")

    @m.pull("lang.user", Summary)
    async def get_summary(self, target: Selector) -> Summary:
        console = self.account.client.storage.current_user
        return Summary(console.nickname, console.nickname)



from __future__ import annotations

from dataclasses import asdict
from typing import TYPE_CHECKING

from nonechat.message import Element
from nonechat.message import Emoji as CslEmoji
from nonechat.message import Markdown as CslMarkdown
from nonechat.message import Markup as CslMarkup
from nonechat.message import Text as CslText

from avilla.console.element import Emoji, Markdown, Markup
from avilla.core.elements import Text
from avilla.core.ryanvk.collector.account import AccountCollector
from avilla.core.ryanvk.descriptor.message.serialize import MessageSerialize

if TYPE_CHECKING:
    from ...account import ConsoleAccount  # noqa
    from ...protocol import ConsoleProtocol  # noqa

ConsoleMessageSerialize = MessageSerialize[Element]


class ConsoleMessageSerializePerform((m := AccountCollector["ConsoleProtocol", "ConsoleAccount"]())._):
    m.post_applying = True

    # LINK: https://github.com/microsoft/pyright/issues/5409

    @ConsoleMessageSerialize.collect(m, Text)
    async def text(self, element: Text) -> Element:
        return CslText(element.text)

    @ConsoleMessageSerialize.collect(m, Emoji)
    async def emoji(self, element: Emoji) -> Element:
        return CslEmoji(element.name)

    @ConsoleMessageSerialize.collect(m, Markup)
    async def markup(self, element: Markup) -> Element:
        return CslMarkup(**asdict(element))

    @ConsoleMessageSerialize.collect(m, Markdown)
    async def markdown(self, element: Markdown) -> Element:
        return CslMarkdown(**asdict(element))



from __future__ import annotations

from avilla.core.application import Avilla
from avilla.core.protocol import BaseProtocol

from .service import ConsoleService


class ConsoleProtocol(BaseProtocol):
    service: ConsoleService
    name: str

    def __init__(self, name: str = "robot"):
        self.name = name

    @classmethod
    def __init_isolate__(cls):  # ruff: noqa: F401
        # isort: off

        # :: Message
        from .perform.message.deserialize import ConsoleMessageDeserializePerform
        from .perform.message.serialize import ConsoleMessageSerializePerform

        # :: Action
        from .perform.action.message import ConsoleMessageActionPerform
        from .perform.action.activity import ConsoleActivityActionPerform
        from .perform.action.profile import ConsoleProfileActionPerform

        # :: Event
        from .perform.event.message import ConsoleEventMessagePerform

    def ensure(self, avilla: Avilla):
        self.avilla = avilla
        self.service = ConsoleService(self)
        avilla.launch_manager.add_component(self.service)



from __future__ import annotations

import sys
from typing import TYPE_CHECKING

from loguru import logger
from nonechat import Backend, Frontend
from nonechat.info import Event

from avilla.core.ryanvk.staff import Staff
from avilla.standard.core.account import (
    AccountAvailable,
    AccountRegistered,
    AccountUnavailable,
)

from ..core.account import AccountInfo
from .account import PLATFORM, ConsoleAccount

if TYPE_CHECKING:
    from .service import ConsoleService


class AvillaConsoleBackend(Backend):
    account: ConsoleAccount
    _service: ConsoleService

    def __init__(self, app: Frontend):
        super().__init__(app)
        self._stderr = sys.stdout
        self._logger_id: int | None = None
        self._should_restore_logger: bool = False

    def set_service(self, service: ConsoleService):
        self._service = service

    def on_console_load(self):
        logger.remove()
        self._should_restore_logger = True
        self.account = ConsoleAccount(self._service.protocol)
        self._logger_id = logger.add(self.frontend._fake_output, level=0, diagnose=False)
        self._service.protocol.avilla.accounts[self.account.route] = AccountInfo(
            self.account.route, self.account, self._service.protocol, PLATFORM
        )
        self._service.protocol.avilla.broadcast.postEvent(
            AccountRegistered(self._service.protocol.avilla, self.account)
        )

    def on_console_mount(self):
        self.account.status.enabled = True
        self._service.protocol.avilla.broadcast.postEvent(AccountAvailable(self._service.protocol.avilla, self.account))

    def on_console_unmount(self):
        if self._logger_id is not None:
            logger.remove(self._logger_id)
            self._logger_id = None
        if self._should_restore_logger:
            logger.add(
                self._stderr,
                backtrace=True,
                diagnose=True,
                colorize=True,
            )
            self._should_restore_logger = False
        self.account.status.enabled = False
        del self._service.protocol.avilla.accounts[self.account.route]
        self._service.protocol.avilla.broadcast.postEvent(
            AccountUnavailable(self._service.protocol.avilla, self.account)
        )
        logger.success("Console exit.")
        logger.warning("Press Ctrl-C for Application exit")

    async def post_event(self, event: Event):
        res = await Staff.focus(self.account).parse_event(event.type, event)
        if res == "non-implemented":
            logger.warning(f"received unsupported event {event.type}: {event}")
            return
        elif res is not None:
            await self._service.protocol.post_event(res)
